#version 330 core

// Interpolated values from the vertex shaders
in vec2 UV;
in vec3 Position_worldspace;
in vec3 Normal;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

in vec3 LightDirection_tangentspace;
in vec3 EyeDirection_tangentspace;

in vec4 FragPosLightSpace;

// Ouput data
layout(location = 0) out vec3 color;

// Values that stay constant for the whole mesh.
uniform sampler2D DiffuseTextureSampler;
uniform sampler2D NormalTextureSampler;
uniform sampler2D SpecularTextureSampler;
uniform sampler2DShadow shadowMap;
uniform mat4 V;
uniform mat4 M;
uniform mat3 MV3x3;
uniform vec3 LightPosition_worldspace;
uniform vec3 lightPos;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;
    float closestDepth = texture(shadowMap, projCoords); 
    float currentDepth = projCoords.z;
    vec3 normal = normalize(Normal);
    vec3 lightDir = normalize(lightPos - Position_worldspace);
	float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005); 
	float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;  

	if(projCoords.z > 1.0)
        shadow = 0.0;

    return shadow;
}

void main(){
	vec3 LightColor = vec3(1,1,1);
	float LightPower = 80.0;
	
	// Material properties
	vec3 MaterialDiffuseColor = texture2D( DiffuseTextureSampler, UV ).rgb;
	vec3 MaterialAmbientColor = 0.6 * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = texture2D( SpecularTextureSampler, UV ).rgb * 0.3;

	// Local normal, in tangent space. V tex coordinate is inverted because normal map is in TGA (not in DDS) for better quality
	vec3 TextureNormal_tangentspace = normalize(texture2D( NormalTextureSampler, UV ).rgb*2.0 - 1.0);
	//vec2(UV.x,-UV.y)

	// Distance to the light
	float distance = length( LightPosition_worldspace - Position_worldspace );

	// Normal of the computed fragment, in camera space
	vec3 n = TextureNormal_tangentspace;
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize(LightDirection_tangentspace);
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );

	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_tangentspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	// Calculate shadow 
	bool shadows = true;
	float shadow = shadows ? ShadowCalculation(FragPosLightSpace) : 0.0;                      
    shadow = min(shadow, 0.75);
	color = 
		// Ambient : simulates indirect lighting
		MaterialAmbientColor +
		// Diffuse : "color" of the object
		((1.0 - shadow) * MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance)) +
		// Specular : reflective highlight, like a mirror
		((1.0 - shadow) * MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5) / (distance*distance));


}